from functools import wraps
import inspect

def deserialize_event(contract_class):
    """
    Decorator to deserialize event data into an event contract class instance.
    Works with both sync and async callback functions.
    """
    def decorator(callback_fn):
        # Check if the callback is an async function
        if inspect.iscoroutinefunction(callback_fn):
            @wraps(callback_fn)
            async def async_wrapper(msg_id, event_data):
                try:
                    obj = contract_class(**event_data)
                except Exception as e:
                    print(f"Deserialization failed for {contract_class.__name__}: {e}")
                    return
                return await callback_fn(obj)
            return async_wrapper
        else:
            @wraps(callback_fn)
            def sync_wrapper(msg_id, event_data):
                try:
                    obj = contract_class(**event_data)
                except Exception as e:
                    print(f"Deserialization failed for {contract_class.__name__}: {e}")
                    return
                return callback_fn(obj)
            return sync_wrapper
    return decorator

def to_db_repr(string):
    return string.replace("'", '"').replace("\\", "/")